/*********************************************************************
* Copyright (C) 2010 by Dario Ignacio Aguilera                       *
* dario_21_06@hotmail.com                                            *
*                                                                    *
* This program is free software; you can redistribute it and/or      *
* modify it under the terms of the GNU General Public License        *
* as published by the Free Software Foundation; either version 2     *
* of the License, or (at your option) any later version.             *
*                                                                    *
* This program is distributed in the hope that it will be useful,    *
* but WITHOUT ANY WARRANTY; without even the implied warranty of     *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      *
* GNU General Public License for more details.                       *
*                                                                    *
* You should have received a copy of the GNU General Public License  *
* along with this program; if not, write to the                      *
* Free Software Foundation, Inc.                                     *
* 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.      *
**********************************************************************/

#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>

class QAction;
class QMenu;
class QScrollArea;
class QAbstractScrollArea;
class QToolBar;
class QSlider;
class QMovie;
class QFileInfo;
class QPaintEvent;
class QMouseEvent;
class ZoomWidget;
class ImageWidget;
class Settings;

enum ImageType{
    Static, //this is for static images, such as jpeg's or not animated gif ones
    Dynamic, //animated images, such as animated gif or png
    Webp //needed becouse qt doesn't handle this yet
};

class MainWindow : public QMainWindow
{
  Q_OBJECT

public:
  /**
   * constructor
   */
  MainWindow();
  
  /**
    * Load app command from command line
    * @param d is the list of the parameters passed
    */
  void loadAppCommands(const QStringList);

protected:
  void resizeEvent(QResizeEvent *);
  void closeEvent(QCloseEvent *event);
  
private:
  //variables
  Settings *settings;
  ImageType imageType;
  ImageWidget *imageWidget;
  ZoomWidget *zoomWidget;
  QSlider *zoomSlider;
  bool showZoomSlider;
  bool imageSetted;
  bool pixmapChanged;
  int compressLevel;
  QStringList nameFilters;
  QFileInfo *archivo;
  QScrollArea *scrollArea;
  QList<QAction *> actionsList;
  QStringList allActions;
  QStringList iconsName;
  QStringList actionsLoaded;
  QString lastDirUsed;//could i delete it and just use archivo->absoluteFilePath()? nop
  int pathToUse;//0=lastDir, 1=default
  QString defaultPath;
  QStringList files;
  int actualPosition;
  int maxRecentFiles;
  QList<QAction *> recentFilesAct;
  QStringList recentFilesPath;

  //toolbar and menus
  QToolBar *mainToolBar;
  QMenu *fileMenu;
  QMenu *viewMenu;
  QMenu *editMenu;
  QMenu *goMenu;
  QMenu *imageMenu;
  QMenu *helpMenu;
  QMenu *recentFilesMenu;
  
  //actions
  QAction *openAct;
  QAction *saveAct;
  QAction *exitAct;
  QAction *aboutAct;
  QAction *aboutQtAct;
  QAction *filePropertiesAct;
  QAction *zoomInAct;
  QAction *zoomOutAct;
  QAction *normalSizeAct;
  QAction *adjustSizeAct;
  QAction *rotateLeftAct;
  QAction *rotateRightAct;
  QAction *flipHorizontallyAct;
  QAction *flipVerticallyAct;
  QAction *nextAct;
  QAction *prevAct;
  QAction *goFirstAct;
  QAction *goLastAct;
  QAction *openDirAct;
  QAction *showMenuBarAct;
  QAction *setTBMovableAct;
  QAction *configAct;
  
  //functions
  QString getLastDir() const;
  void exploreFolder(const QString="");
  void createActions();
  void createMenus();
  void createToolBar();
  void setNameFilters();
  void determinateImageType();
  void openImage();
  void updateActions();
  bool goToImage(int);
  void getColorFromSettings(QStringList);
  
  /**
    *creating actions for the recent files
    */
  void createRFActs();
  
  /**
    *this method is called when i open an image, to add it to the recent files
    */
  void pushNewFile();
  void setShortcuts(bool, QStringList, QStringList);
  
  /**
    * configure tool bar
    *
    * @param sl is the list of the actions to add to the toolbar
    * @param a is the tool bar area
    * @param s the tool bar buttom style
    * @param v tool bar visible or not
    * @param zl show zoom slider
    */
  void configureToolBar(QStringList, Qt::ToolBarArea, Qt::ToolButtonStyle, bool, bool);
  
private slots:
  
  /**
    *this methos is for moving the image with the mouse
    * @param e is the QPoint value from mousePressEvent
    * @param d is the QPoint value from mouseMoveEvent
    */
  void moveWidget(QPoint, QPoint);
  void updateWindowTitle();
  
  /**
    *this is for when the image is rotated and the button fixed size
    *is checked
    */
  void imageTranformated();
  void open();
  void save();
  void next();
  void previous();
  void goFirst();
  void goLast();
  void openDir();
  void openRecentFile();
  void openStatic();
  void openDynamic();
  void fileProperties();
  void zoomIn();
  void zoomOut();
  void wheelZoom(int);
  void normalSize();
  void adjustSizeSlot();
  void rotateLeft();
  void rotateRight();
  void flipHorizontally();
  void flipVertically();
  void about();
  void couldOpen(bool);
  void loadSettings();
  void saveSettings();
  void showMenuBar();
  void setToolBarMovable();
  
  /**
    * configuration dialog
    */
  void configureProgram();
  inline void pixmapChangedSlot(){pixmapChanged = true;}
};

#endif
