/*********************************************************************
* Copyright (C) 2010 by Dario Ignacio Aguilera                       *
* dario_21_06@hotmail.com                                            *
*                                                                    *
* This program is free software; you can redistribute it and/or      *
* modify it under the terms of the GNU General Public License        *
* as published by the Free Software Foundation; either version 2     *
* of the License, or (at your option) any later version.             *
*                                                                    *
* This program is distributed in the hope that it will be useful,    *
* but WITHOUT ANY WARRANTY; without even the implied warranty of     *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      *
* GNU General Public License for more details.                       *
*                                                                    *
* You should have received a copy of the GNU General Public License  *
* along with this program; if not, write to the                      *
* Free Software Foundation, Inc.                                     *
* 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.      *
**********************************************************************/

#include <QtGui>
#include "toolbaredit.h"

ToolBarEdit::ToolBarEdit(QWidget* parent, Qt::WFlags fl) : QWidget(parent, fl)
{
  QLabel *etb = new QLabel;
  etb->setText("Editar barra de herramientas");
  
  avaibleActions = new QListWidget;
  //avaibleActions->setAcceptDrops(true);
  //avaibleActions->setDragEnabled(true);
  actualActions =new QListWidget;
  actualActions->setSelectionMode(QAbstractItemView::MultiSelection);
  //actualActions->setAcceptDrops(true);
  //actualActions->setDragEnabled(true);
  
  connect(this, SIGNAL(actionListAdded()), this, SLOT(configureActionList()));
  connect(avaibleActions, SIGNAL(itemSelectionChanged()), this, SLOT(enableActionsA()));
  connect(actualActions, SIGNAL(itemSelectionChanged()), this, SLOT(enableActionsB()));
  
  createActionsButtons();
    
  mainLayout = new QHBoxLayout;
  mainLayout->addWidget(avaibleActions);
  mainLayout->addLayout(buttonsLayout);
  mainLayout->addWidget(actualActions);
  
  /*QHBoxLayout *asd = new QHBoxLayout;
  asd->addStretch();
  asd->addWidget(getActionsListButton);*/
  
  vLayout = new QVBoxLayout;
  vLayout->addWidget(etb);
  vLayout->addLayout(mainLayout);
  //vLayout->addLayout(asd);
  
  setLayout(vLayout);  
}

QStringList ToolBarEdit::getActionsList()
{
  QStringList actions;
  for(int i=0; i<actualActions->count(); i++){
    actions<<actualActions->item(i)->text();
  }
  actions.replaceInStrings("--- ", "");
  actions.replaceInStrings(" ---", "");
  return actions;
}

void ToolBarEdit::createActionsButtons()
{  
  /*getActionsListButton = new QPushButton(tr("Get actions list"));
  getActionsListButton->setIcon(QIcon::fromTheme("tools-wizard"));
  getActionsListButton->setEnabled(true);
  connect(getActionsListButton, SIGNAL(clicked()), this, SLOT(getActionsList()));*/
  
  addActionButton = new QToolButton;
  addActionButton->setIcon(QIcon::fromTheme("go-next"));
  addActionButton->setToolTip(tr("Add action"));
  addActionButton->setEnabled(false);
  connect(addActionButton, SIGNAL(clicked()), this, SLOT(addAction()));
  
  removeActionButton = new QToolButton;
  removeActionButton->setIcon(QIcon::fromTheme("go-previous"));
  removeActionButton->setToolTip(tr("Remove action"));
  removeActionButton->setEnabled(false);
  connect(removeActionButton, SIGNAL(clicked()), this, SLOT(removeAction()));
  
  moveUpActionButton = new QToolButton;
  moveUpActionButton->setIcon(QIcon::fromTheme("go-up"));
  moveUpActionButton->setToolTip(tr("Move action up"));
  moveUpActionButton->setEnabled(false);
  connect(moveUpActionButton, SIGNAL(clicked()), this, SLOT(moveUpAction()));
  
  moveDownActionButton = new QToolButton;
  moveDownActionButton->setIcon(QIcon::fromTheme("go-down"));
  moveDownActionButton->setToolTip(tr("Move action down"));
  moveDownActionButton->setEnabled(false);
  connect(moveDownActionButton, SIGNAL(clicked()), this, SLOT(moveDownAction()));
  
  addSeparatorButton = new QToolButton;
  addSeparatorButton->setIcon(QIcon::fromTheme("list-add"));
  addSeparatorButton->setToolTip(tr("Add separator"));
  addSeparatorButton->setEnabled(false);
  addSeparatorButton->setAutoRaise(true);
  connect(addSeparatorButton, SIGNAL(clicked()), this, SLOT(addSeparatorAction()));
  
  buttonsLayout = new QGridLayout;
  buttonsLayout->setSpacing( 0 );
  buttonsLayout->setRowStretch( 0, 10 );
  buttonsLayout->addWidget(moveUpActionButton, 1, 1, Qt::AlignHCenter);
  buttonsLayout->addWidget(removeActionButton, 2, 0, Qt::AlignHCenter);
  buttonsLayout->addWidget(addSeparatorButton, 2, 1, Qt::AlignHCenter);
  buttonsLayout->addWidget(addActionButton, 2, 2, Qt::AlignHCenter);
  buttonsLayout->addWidget(moveDownActionButton, 3, 1, Qt::AlignHCenter);
  buttonsLayout->setRowStretch( 4, 10 );
  
}

void ToolBarEdit::addSeparatorAction()
{
  int i=actualActions->currentRow();
  switch(changeSeparatorButton()){
    case 0:
      if(i==-1){
	i=actualActions->count();
      }
      actualActions->insertItem(i+1, new QListWidgetItem(QString("--- %1 ---").arg(tr("separator"))));
      actualActions->setCurrentRow(i+1);
      break;
    case 1:
      actualActions->takeItem(i);
      break;
  }
  emit actionsListChanged();
}

void ToolBarEdit::addAction()
{
  /*qDebug()<<"Agregar accion"<<avaibleActions->currentRow()+1
	  <<":"<<avaibleActions->currentItem()->text();*/
  int i=actualActions->currentRow()+1;
  if(i==0){i=actualActions->count();}
  for(int i=0; i<actualActions->count(); i++){
      if(actualActions->item(i)->text() ==
         avaibleActions->item(avaibleActions->currentRow())->text()){
         return;
      }
  }
  actualActions->insertItem(i, avaibleActions->item(avaibleActions->currentRow())->clone());
  actualActions->setCurrentRow(i);
  removeActionButton->setEnabled(true);
  emit actionsListChanged();
}

void ToolBarEdit::removeAction()
{
    if(actualActions->count() == 1){
        actualActions->clear();
        removeActionButton->setEnabled(false);
        return;
    }
    QListWidgetItem *g = actualActions->takeItem(actualActions->currentRow());
    g->~QListWidgetItem();
    emit actionsListChanged();
    //actualActions->removeItemWidget(actualActions->item(actualActions->currentRow()));
    /*qDebug()<<"Quitar accion"<<actualActions->currentRow()+1
            <<":"<<actualActions->currentItem()->text();*/
    /*bool add=true;
    for(int i=0; i<avaibleActions->count(); i++){
        if(avaibleActions->item(i)->text() == actualActions->item(actualActions->currentRow())->text()){
            add = false;
            i = avaibleActions->count();
        }
    }
    if(add){
        avaibleActions->addItem(actualActions->takeItem(actualActions->currentRow()));
        emit actionsListChanged();
    }*/
}

void ToolBarEdit::moveUpAction()
{
  int g=actualActions->currentRow();
  actualActions->insertItem(g-1, actualActions->takeItem(g));
  actualActions->setCurrentRow(g-1);
  emit actionsListChanged();
}

void ToolBarEdit::moveDownAction()
{
  int g=actualActions->currentRow();
  actualActions->insertItem(g+1, actualActions->takeItem(g));
  actualActions->setCurrentRow(g+1);
  emit actionsListChanged();
}

void ToolBarEdit::enableActionsA()
{
  //qDebug()<<"Item seleccionado:"<<avaibleActions->currentRow()+1;
  addActionButton->setEnabled(avaibleActions->count()-1!=0);
}

int ToolBarEdit::changeSeparatorButton()
{
  //qDebug()<<actualActions->currentItem()->text();
  if(actualActions->currentItem()->text()!=QString("--- %1 ---").arg(tr("separator"))){
    return 0;
  }
  if(actualActions->currentItem()->text()==QString("--- %1 ---").arg(tr("separator"))){
    return 1;
  }
  return 2;
}

void ToolBarEdit::enableActionsB()
{
    //qDebug()<<"cantidad:"<<actualActions->count();
    //qDebug()<<"posicopm:"<<actualActions->currentRow();
    //qDebug()<<"Item seleccionado:"<<actualActions->currentRow()+1;
    removeActionButton->setEnabled(actualActions->count()-1!=0);
    moveDownActionButton->setEnabled(false);
    moveUpActionButton->setEnabled(false);
    addSeparatorButton->setEnabled(true);
    switch(changeSeparatorButton()){
      case 0:
        addSeparatorButton->setIcon(QIcon::fromTheme("list-add"));
        addSeparatorButton->setToolTip(tr("Add separator"));
        break;
      case 1:
        addSeparatorButton->setIcon(QIcon::fromTheme("list-remove"));
        addSeparatorButton->setToolTip(tr("Remove separator"));
        break;
    }
    if(actualActions->currentRow()>=1){
      moveUpActionButton->setEnabled(true);
    }
    if(actualActions->currentRow() < actualActions->count()-1){
      moveDownActionButton->setEnabled(true);
    }
}

void ToolBarEdit::setActionsList(QStringList d1, QStringList d2)
{
  allActions = d1;
  allActions.replaceInStrings("&", "");
  actionsUsed = d2;
  /*if(false){
  qDebug()<<"------------Allactions---------------";
  for(int j=0; j<allActions.size(); j++){
    qDebug()<<allActions.at(j);
  }
  qDebug()<<"-------------------------------------";
  qDebug()<<"------------actionsused--------------";
  for(int i=0; i<actionsUsed.size(); i++){
    qDebug()<<actionsUsed.at(i);
  }
  qDebug()<<"-------------------------------------";
  }*/
  emit actionListAdded();
}

void ToolBarEdit::configureActionList()
{
  seticons=false;
  auActualItems();
}

void ToolBarEdit::auActualItems()
{
  avaibleActions->clear();
  actualActions->clear();
  actualActions->setDragDropMode(QAbstractItemView::InternalMove);
  QListWidgetItem *item;
  for(int i=0; i<allActions.size(); i++){
    item = new QListWidgetItem(allActions.at(i));
    //qDebug()<<icons;
    if(seticons && icons.at(i) != "none"){
        item->setIcon(QIcon::fromTheme(icons.at(i)));
    }
    else{
      item->setText(QString("---  %1").arg(item->text()));
    }
    avaibleActions->addItem(item);
  }      
  for(int i=0; i<actionsUsed.size(); i++){
    item = new QListWidgetItem(actionsUsed.at(i));
    actualActions->addItem(item);
  }
  
  QList<int> apos;
  QList<int> bpos;
  actionsUsed.replaceInStrings(tr("separator"), QString("--- %1 ---").arg(tr("separator")));
  for(int j=0; j<actionsUsed.size(); j++){
   bool add=false;
   for(int i=0; i<avaibleActions->count(); i++){
      if(avaibleActions->item(i)->text() == actionsUsed.at(j) && !add){
	//qDebug()<<"accion:"<<avaibleActions->item(i)->text();
	apos<<i;
	bpos<<j;
	add=true;
      }
   }
  }
  for(int i=0; i<apos.size(); i++){
    actualActions->item(bpos.at(i))->setIcon(avaibleActions->item(apos.at(i))->icon());
  }
  /*if(false){
  qDebug()<<"apos:"<<apos;
  qDebug()<<"bpos:"<<bpos;
  qDebug()<<apos.size();
  qDebug()<<bpos.size();}*/
}

void ToolBarEdit::setIcons(QStringList d)
{
  icons=d;
  seticons=!d.isEmpty();
  //qDebug()<<"seticons:"<<seticons;
  auActualItems();
}
